#include "nrf.h"
#include "nrf_gpiote.h"
#include "nrf_gzll.h"
#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include "nrf_nvmc.h"
#include "gzl_config.h"
#include "timer_config.h"
#include "nrf_gpio.h"
#include "app_uart.h"
#include "nrf_delay.h"
#include "app_error.h"

#define DATAPIPE 1 ///< We use pipe 0 in this example
#define PAIRPIPE 0
#define TX_PAYLOAD_LENGTH 1 ///< We use 1 byte payload length when transmitting
#define LEDA 24
#define LEDB 19
#define AUTHBUTTON 16
#define PARXPIN 17

#define RX_PIN_NUMBER 28
#define TX_PIN_NUMBER	29
#define RTS_PIN_NUMBER	24
#define CTS_PIN_NUMBER	25
#define UART_TX_BUF_SIZE 256                        /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE 1   



uint16_t PWMWITH[10] = {1500,1500,1500,1500,1500,1500,1500,1500,1500,1500};

uint32_t pg_size;
uint32_t pg_num;
uint32_t nvmcaddr;
bool pairedflag = 0;
uint32_t HOSTADDR = 0;
bool popok=0;
static uint8_t data_payload[NRF_GZLL_CONST_MAX_PAYLOAD_LENGTH]={0};  ///< Placeholder for data payload received from host. 
static uint8_t data_ack[16]={0x23};
static uint8_t pair_ack[9]={0xaa};
uint8_t Lostcount =0;
uint8_t Connectcount =0;
bool Lostflag =0;
bool Connectflag =0;

uint8_t DATAPIPEADDR = 0;


nrf_gzll_host_rx_info_t my_host_rx_info;

void uart_error_handle(app_uart_evt_t * p_event)
{

}
static void uart_init(void)
{
    uint32_t                     err_code;
    const app_uart_comm_params_t comm_params =
    {
        RX_PIN_NUMBER,
        TX_PIN_NUMBER,
        RTS_PIN_NUMBER,
        CTS_PIN_NUMBER,
        APP_UART_FLOW_CONTROL_DISABLED,
        false,
        UART_BAUDRATE_BAUDRATE_Baud115200
    };

    APP_UART_FIFO_INIT( &comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_error_handle,
                       APP_IRQ_PRIORITY_LOW,
                       err_code);
    APP_ERROR_CHECK(err_code);
}


void gpio_init()
{
	nrf_gpio_cfg_output(LEDA);
	nrf_gpio_cfg_output(LEDB);
	nrf_gpio_cfg_output(PARXPIN);
	nrf_gpio_cfg_input(AUTHBUTTON,NRF_GPIO_PIN_PULLUP);
	nrf_gpio_pin_set(PARXPIN);
#ifdef PPMOUTPUT
	nrf_gpio_cfg_input(PWMCH2,NRF_GPIO_PIN_PULLUP);
	nrf_gpio_cfg_input(PWMCH3,NRF_GPIO_PIN_PULLUP);
	nrf_gpio_cfg_input(PWMCH4,NRF_GPIO_PIN_PULLUP);
	nrf_gpio_cfg_input(PWMCH5,NRF_GPIO_PIN_PULLUP);
	nrf_gpio_cfg_input(PWMCH6,NRF_GPIO_PIN_PULLUP);
	nrf_gpio_cfg_input(PWMCH7,NRF_GPIO_PIN_PULLUP);
	nrf_gpio_cfg_output(PWMCH1);	
#else
	nrf_gpio_port_dir_set(NRF_GPIO_PORT_SELECT_PORT0,NRF_GPIO_PORT_DIR_OUTPUT);
#endif
}

int main()
{
	DATAPIPEADDR = (NRF_FICR->DEVICEID[1]) & 0x000000ff;
	gpio_init();
	uart_init();
	nrf_gzll_init(NRF_GZLL_MODE_HOST); 
//	nrf_gzll_set_max_tx_attempts(5);
	nrf_gzll_set_tx_power(NRF_GZLL_TX_POWER_4_DBM);
	rc_gzll_set_addr();
	nrf_gzll_set_base_address_1(NRF_FICR->DEVICEID[0]);
	nrf_gzll_set_address_prefix_byte(DATAPIPE,DATAPIPEADDR);
	rc_gzll_set_freqhop();
	nrf_gzll_set_datarate(GZLSPEED);	
	nrf_gzll_enable();
	printf("Host Start:\n\r");
	printf("HWID:%x %x\n\r",NRF_FICR->DEVICEID[1],NRF_FICR->DEVICEID[0]);
	
	pg_size = NRF_FICR->CODEPAGESIZE;
	pg_num  = NRF_FICR->CODESIZE - 1;   
	nvmcaddr=(uint32_t )(pg_size*pg_num);
	
	nrf_gpio_port_dir_set(NRF_GPIO_PORT_SELECT_PORT0,NRF_GPIO_PORT_DIR_OUTPUT);
	timer0_init(); 
	timer1_init();
	NVIC_EnableIRQ(TIMER0_IRQn);
	NVIC_EnableIRQ(TIMER1_IRQn);
	__enable_irq();
	while(1)
	{	

	}
}
void nrf_gzll_host_rx_data_ready(uint32_t pipe, nrf_gzll_host_rx_info_t rx_info) 
{
	uint32_t data_payload_length =20;
	//printf("UNKNOW!!\n\r");
//	printf("Rssi:%d\n\r",rx_info.rssi);	
	popok = nrf_gzll_fetch_packet_from_rx_fifo(pipe, data_payload, &data_payload_length);
	Connectcount++;
	if(Connectcount == 100)
	{	
		Connectcount = 0;
		Lostcount = 0;
		Lostflag = 0;
	}
//	printf("Freq:%d\n\r",NRF_RADIO->FREQUENCY);
	if((pipe==PAIRPIPE) && (data_payload[0]==0xaa) && (data_payload[1]==0xaa) && (data_payload[2]==0xaa) && (data_payload[3]==0xaa) )//&& (rx_info.rssi>(-43)))// (nrf_gpio_pin_read(AUTHBUTTON)==0))  //rx_info.rssi>(-35)) //?????
	{
			HOSTADDR = NRF_FICR->DEVICEID[0];
			printf("Receive pair request!!\n\r");
			memset(pair_ack,0xbb,9);
			pair_ack[4] =	(HOSTADDR&0x000000ff);
			pair_ack[5] =	(HOSTADDR&0x0000ff00)>>8;
			pair_ack[6] =	(HOSTADDR&0x00ff0000)>>16;
			pair_ack[7] =	(HOSTADDR&0xff000000)>>24;
			pair_ack[8] =	DATAPIPEADDR;
	
			nrf_gzll_add_packet_to_tx_fifo(PAIRPIPE,pair_ack,9);
			printf("Send pair information to device!!\n\r");
			printf("ADDR: %d",HOSTADDR);
			pairedflag = true;		
		}
		else if(pipe == DATAPIPE)
		{		
			PWMWITH[0] = data_payload[0]; 
			PWMWITH[0] = (PWMWITH[0]<<8) | (data_payload[1]); 
			PWMWITH[1] = data_payload[2]; 
			PWMWITH[1] = (PWMWITH[1]<<8) | (data_payload[3]); 
			PWMWITH[2] = data_payload[4]; 
			PWMWITH[2] = (PWMWITH[2]<<8) | (data_payload[5]); 	
			PWMWITH[3] = data_payload[6]; 
			PWMWITH[3] = (PWMWITH[3]<<8) | (data_payload[7]); 
			
			PWMWITH[4] = data_payload[8]; 
			PWMWITH[4] = (PWMWITH[4]<<8) | (data_payload[9]); 
			PWMWITH[5] = data_payload[10]; 
			PWMWITH[5] = (PWMWITH[5]<<8) | (data_payload[11]); 
			PWMWITH[6] = data_payload[12]; 
			PWMWITH[6] = (PWMWITH[6]<<8) | (data_payload[13]); 
			PWMWITH[7] = data_payload[14]; 
			PWMWITH[7] = (PWMWITH[7]<<8) | (data_payload[15]);		
			PWMWITH[8] = data_payload[16]; 
			PWMWITH[8] = (PWMWITH[8]<<8) | (data_payload[17]); 
			PWMWITH[9] = data_payload[18]; 
			PWMWITH[9] = (PWMWITH[9]<<8) | (data_payload[19]);

			data_ack[0] = 0x4F;
			data_ack[1] = 0x4B;
			data_ack[3] = rx_info.rssi;
			data_ack[2] = rx_info.rssi>>8;
			nrf_gzll_add_packet_to_tx_fifo(DATAPIPE,data_ack,9);
			nrf_gpio_pin_toggle(LEDA);
		//	printf("Host PIPE%d WORK SUCCESS!\n\r",pipe);	
			
		}
}

void nrf_gzll_disabled() {}
void nrf_gzll_device_tx_success(uint32_t pipe, nrf_gzll_device_tx_info_t tx_info) {}
void nrf_gzll_device_tx_failed(uint32_t pipe, nrf_gzll_device_tx_info_t tx_info)
{}
